library staffino;

import 'dart:html';
import 'dart:async';
import 'package:angular/angular.dart';
import 'package:angular/animate/module.dart';
import 'package:angular/application_factory.dart';
import 'package:logging/logging.dart';
import 'package:yaml/yaml.dart';

import 'package:staffino_frontend/routes.dart';
import 'package:staffino_frontend/models/models.dart';
import 'package:staffino_frontend/components/components.dart';
import 'package:staffino_frontend/client_storage/client_storage.dart';
import 'package:staffino_frontend/formatters/formatters.dart';
import 'package:staffino_frontend/filters/filters.dart';
import 'package:staffino_frontend/decorators/decorators.dart';
import 'package:staffino_frontend/helpers/helpers.dart';
import 'package:staffino_frontend/services/services.dart';
import 'package:staffino_frontend/my_exception_handler.dart';

void main() {
  String environment = getEnvironment();
  addEmptyHashToUrl();
  setupLogging(environment);


  StaffinoModule staffino = new StaffinoModule(environment);
  staffino.setUp().then((_) {
    Application app = applicationFactory()
      .addModule(staffino)
      .rootContextType(BgController);

    AppConfig.app = app;
    Injector injector = app.run();

    NetModel.setupConn(injector.get(Connection));

    AppConfig.injector = injector;
  });
}

String getEnvironment() {
  switch (window.location.hostname) {
    //case 'localhost':
    //case 'staffino-development.dev':
    //case 'app-development.staffino.com':
    //  return Environments.STAGING;

    //case 'app-staging.staffino.com':
    //  return Environments.STAGING;

    default:
      return Environments.PRODUCTION;
  }
}

void addEmptyHashToUrl() {
  if (window.location.hash.isEmpty) {
    String urlWithHash = window.location.toString();
    urlWithHash += '#/';
    window.history.replaceState({}, document.title, urlWithHash);
  }
}

void setupLogging(String env) {
  if (env == Environments.PRODUCTION) {
    Logger.root.level = Level.OFF;
  } else {
    Logger.root.level = Level.ALL;
    Logger.root.onRecord.listen((LogRecord rec) {
      if (rec.loggerName.startsWith('_')) {
        print('${rec.level} ${rec.loggerName}: ${rec.message}');
      } else if (rec.level == Level.FINEST) {
        print('Angular: ${rec.message}');
      }
    });
  }
}

class StaffinoModule extends Module {
  String _env;

  StaffinoModule(String this._env) {
    Binding.printInjectWarning = false;

    install(new AnimationModule());

    bind(NgRoutingUsePushState, toValue: new NgRoutingUsePushState.value(false));

    bind(MyExceptionHandler);
    bind(ExceptionHandler, toInstanceOf: MyExceptionHandler);
    bind(Connection);
    bind(Http);

    bind(AvatarComponent);
    bind(FileUploadComponent);
    bind(SidebarMenu);
    bind(TopMenu);
    bind(ErrorMsgComponent);
    bind(MomentComponent);
    bind(DropdownComponent);
    bind(FormIndicator);
    bind(Loader);
    bind(DonutGraph);
    bind(DialGraph);
    bind(DialSmallGraph);
    bind(LineGraph);
    bind(StackGraph);
    bind(AngryFace);
    bind(Crash);
    bind(SortableStaff);
    bind(SortableVenues);
    bind(SortableStaffSingle);
    bind(SortableManagersSingle);
    bind(SortableProductsSingle);
    bind(SortableManagers);
    bind(SortableProducts);
    bind(SortableRegions);
    bind(DateRange);
    bind(SmartFilter);
    bind(MemberCard);
    bind(SliderToggle);
    bind(WordCloud);
    bind(ImageEditor);
    bind(FollowUp);

    bind(BannerDownloadApp);

    bind(BgController);
    bind(LoginController);
    bind(InController);
    bind(ViewController);
    bind(ProfileController);
    bind(ProfileOrganizationUnitController);
    bind(NotificationsController);
    bind(NotRespondedConroller);
    bind(FeedbacksController);
    bind(ResolutionCenterController);
    bind(InternalDiscussionController);
    bind(TeamController);
    bind(ImportMember);
    bind(VenueController);
    bind(NewStatisticsController);
    bind(StatisticsController);
    bind(VenueReloadController);
    bind(RewardController);
    bind(BillingController);
    bind(ChoosePlanController);
    bind(PaymentHistoryController);
    bind(InvoicesController);
    bind(CreditCardsController);
    bind(BillingInfoController);
    bind(ClientDatabase);
    bind(ClientDatabasesCollection);
    bind(ClientDatabaseImport);
    bind(ClientDatabaseImportUpload);
    bind(ClientDatabaseRequest);
    bind(ClientDatabaseStatistics);
    bind(FGController);
    bind(FGControllerBg);
    bind(FGQuestionsController);
    bind(FGQuestionController);
    bind(FGPreviewController);
    bind(FGResultsController);
    bind(FGStatisticsController);
    bind(FGSettingsController);
    bind(FGCustomersController);
    bind(FGEditPreset);
    bind(FGQuestionnaireModal);
    bind(RFController);
    bind(RFControllerBg);
    bind(RFInteractionsController);
    bind(RFCustomersController);
    bind(RFVenueIdController);
    bind(RFStatisticsController);
    bind(RFSettingsController);
    bind(RFTemplatesController);
    bind(Agreement);
    bind(StatisticsResponse);

    bind(EditPreset);
    bind(TemplateEditor);
    bind(RelativeDateFormatter);
    bind(FilterFeelingFormatter);
    bind(DurationFormatter);
    bind(IfEmptyFormatter);
    bind(FilterEmploymentVisibilityFormatter);
    bind(FilterRoleFormatter);
    bind(MonthFormatter);
    bind(ToLocalTimeZone);
    bind(PositionFormatter);
    bind(VenueFormatter);
    bind(GenericStaffAwareFirstName);
    bind(GenericStaffAwareLastName);
    bind(GenericStaffAwareFullName);
    bind(CreditCardFormatter);
    bind(Nl2br);
    bind(ManagerPositionFormatter);
    bind(DropdownVenueImageFormatter);
    bind(QuestionFormatter);
    bind(FGQuestionFormatter);
    bind(ClientDatabaseFormatter);

    bind(SameAsValidator);
    bind(RequiredIfValidator);
    bind(FileMaxSizeValidator);
    bind(NgBindOnce);
    bind(NgIfOnce);
    bind(DisHref);
    bind(NgPlaceholder);
    bind(NgPermission);
    bind(NgSubmitPermissionsAware);
    bind(NgBindHtmlUnsafe);
    bind(NgBindHtmlBrOnly);

    bind(TeamFilter);
    bind(IsEmptyPredicate);
    bind(I18nFilter);
    bind(MyCurrency);

    bind(AuthService);
    bind(PollingUnreadFeedbackService);
    bind(PollingUnreadFgAnswersService);
    bind(PollingUnrespondedFeedbackService);
    bind(FeedbacksFilterMemory);
    bind(TeamFilterMemory);
    bind(StatisticsFilterMemory);
    bind(RFFilterMemory);
    bind(FGFilterMemory);
    bind(Settings);
    bind(Translator);
    bind(MeService);
    bind(VenueService);
    bind(LogoutService);
    bind(PermissionsService);
    bind(PositionsService);
    bind(FeedbackService);
    bind(FocusGroupService);

    bind(ClientStorageProvider);

    bind(Routes);
    bind(RouteInitializerFn, toImplementation: Routes);
  }

  Future setUp() =>
    initConfig()
    .then((_) => initI18n())
    .then((_) => this);

  Future initConfig() {
    // cache breaker will set some value
    var $timestamp;
    var dir = (_env == Environments.TESTING) ? '../test/' : '';
    return HttpRequest.getString('${dir}config.yaml?timestamp=${$timestamp}').then((String response) {
      Map config = new Map.from(loadYaml(response));
      AppConfig.init(config, _env);
    });
  }

  Future initI18n() {
    var dir = (_env == Environments.TESTING) ? '../test/' : '';
    return Translator.getLocaleSavedToClient().then((String locale) =>
      Translator.init('${dir}translations', locale)
    );
  }
}